@@ -7,7 +7,7 @@ start_seed = 1000
 number_of_seeds = 10
 
 seeds = list(range(start_seed, start_seed + number_of_seeds))
-sizes = [(1, 1), (2, 2), (3, 3), (2, 6), (6, 2), (5, 5)]
+sizes = [(1, 1), (2, 2), (3, 3), (2, 6), (6, 2), (5, 5), (8, 8)]
 abslimits = [1, 10, 100, 10000]
 
 
@@ -40,7 +40,7 @@ def test_snfproblem_zi(random_seed, size, abslimit):
     w = size[1]
 
     contents = [rand.randint(-abslimit, abslimit) for _ in range(h*w*2)]
-    zi_contents = [zi.ZI(contents[2*i], contents[2*i+1]) for i in range(h*w)]
+    zi_contents = [zi.ZI([contents[2*i], contents[2*i+1]]) for i in range(h*w)]
 
     m = matrix.Matrix(h, w, zi_contents)
 
